#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.helper.container module

This module provides container-related helpers to delete elements or update elements
attributes.
"""

from pyramid.httpexceptions import HTTPForbidden, HTTPInternalServerError

from pyams_security.permission import get_edit_permission


__docformat__ = 'restructuredtext'

from pyams_skin import _


def delete_container_element(request, container_factory=None, ignore_permission=False):
    """Delete container element

    This view is not strictly protected, but:
    - either the function is called from another protected view
    - either the view is checking edit permission from context adapter; if permission can't be
      found, an internal server error is raised!
    If the function is called from another unprotected view with 'ignore_permission=True',
    it's a configuration error.

    :param request: the current request
    :param container_factory: adapter interface or factory which may be used to access required
        container values
    :param ignore_permission: if False, container's edit permission is checked and an exception is
        raised if request doesn't have required permission; otherwise, no permission is checked.
        This argument should be set to True only when the function is called from another view
        which already checked required permission.
    """
    translate = request.localizer.translate
    # Get object name to be removed
    name = request.params.get('object_name')
    if not name:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("No provided object_name argument!"))
            }
        }
    # Check container factory
    container = request.context
    if container_factory is not None:
        container = container_factory(container)
    # Check container
    if name not in container:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("Given element name doesn't exist!"))
            }
        }
    # Check permission
    context = container[name]
    if not ignore_permission:
        permission = get_edit_permission(request, context, action='delete')
        if permission is None:
            raise HTTPInternalServerError("Missing permission definition!")
        if not request.has_permission(permission, context=context):
            raise HTTPForbidden()
    # Delete element
    del container[name]
    return {
        'status': 'success',
        'message': translate(_("Element was deleted successfully."))
    }


def switch_element_attribute(request, container_factory=None, ignore_permission=False):
    """Switch attribute of a container element

    This view is not strictly protected, but:
    - either the function is called from another protected view
    - either the view is checking edit permission from context adapter; if permission can't be
      found, an internal server error is raised!
    If the function is called from another unprotected view with 'ignore_permission=True',
    it's a configuration error.

    :param request: the current request
    :param container_factory: adapter interface or factory which may be used to access required
        container values
    :param ignore_permission: if False, container's edit permission is checked and an exception is
        raised if request doesn't have required permission; otherwise, no permission is checked.
        This argument should be set to True only when the function is called from another view
        which already checked required permission.
    """
    translate = request.localizer.translate
    # Get object name to be removed
    name = request.params.get('object_name')
    if not name:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("No provided object_name argument!"))
            }
        }
    attribute = request.params.get('attribute_name')
    if not attribute:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("No provided attribute name!"))
            }
        }
    # Check container factory
    container = request.context
    if container_factory is not None:
        container = container_factory(container)
    # Check container
    if name not in container:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("Given element name doesn't exist!"))
            }
        }
    # Check permission
    context = container[name]
    if not ignore_permission:
        permission = get_edit_permission(request, context)
        if permission is None:
            raise HTTPInternalServerError("Missing permission definition!")
        if not request.has_permission(permission, context=context):
            raise HTTPForbidden()
    # Update element
    setattr(context, attribute, not getattr(context, attribute))
    return {
        'status': 'success',
        'message': translate(_("Element was updated successfully.")),
        'state': getattr(context, attribute)
    }



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.table module

This module defines public interfaces of table views.
"""

from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, TextLine

from pyams_skin.interfaces.viewlet import IDropdownMenu
from pyams_template.template import template_config
from pyams_zmi.interfaces import IInnerAdminView

__docformat__ = 'restructuredtext'

from pyams_zmi import _


@template_config(template='templates/table.pt')
@template_config(template='templates/table-empty.pt', name='empty')
class ITableView(Interface):
    """Table view interface"""

    table = Attribute("Inner table instance")


class ITableAdminView(ITableView, IInnerAdminView):
    """Admin table view interface"""

    table_class = Attribute("Inner table class")

    table_label = TextLine(title="Inner table label")


class ITableAttributes(Interface):
    """Table data attributes updater interface"""

    def update_attributes(self, source: dict):
        """Update source settings with new values"""


@template_config(template='templates/inner-table.pt')
@template_config(template='templates/inner-table-empty.pt', name='empty')
class IInnerTable(IContentProvider):
    """Inner admin table view interface"""

    table_class = Attribute("Inner table class")

    table_label = TextLine(title="Inner table label")


@template_config(template='templates/table-multiple.pt')
class IMultipleTableView(IInnerAdminView):
    """Multiple table view"""


class ITableWithActions(Interface):
    """Marker interface for table with inner actions menu"""


class ITableActionsColumnMenu(IDropdownMenu):
    """Table item actions menu"""


@template_config(template='templates/table-switcher.pt')
class ITableGroupSwitcher(Interface):
    """Table group switcher interface"""

    legend = TextLine(title=_("Switcher legend"),
                      required=True)

    minus_class = TextLine(title="Expanded switcher FontAwesome CSS class (without prefix)",
                           default='chevron-down')

    plus_class = TextLine(title="Reduced switcher FontAwesome CSS class (without prefix)",
                          default='chevron-right')

    switcher_mode = Choice(title="Switcher display mode",
                           values=('always', 'never', 'auto'),
                           default='auto')

    state = Attribute("Initial switcher state")


class ITableElementEditor(Interface):
    """Table row element editor marker interface"""

    view_name = TextLine(title="Editor view name",
                         default='properties.html')

    href = TextLine(title="Editor URL")

    modal_target = Bool(title="Modal target?",
                        required=True,
                        default=True)


class IReorderColumn(Interface):
    """Reorder column marker interface"""


class IColumnSortData(Interface):
    """Column with custom sorting data interface"""
    
    def get_sort_value(self, item):
        """Get sort value from line item"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi module

This module defines the global management interface view!
"""

from fanstatic import Library, Resource
from zope.interface import Interface

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_template.template import template_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.view import AdminView


__docformat__ = 'restructuredtext'


library = Library('pyams_zmi', 'resources')

swagger_css = Resource(library, 'css/swagger-ui.min.css')

swagger_ui = Resource(library, 'js/swagger-ui-bundle.min.js',
                      bottom=True)

swagger_presets = Resource(library, 'js/swagger-ui-standalone-preset.min.js',
                           depends=(swagger_ui,),
                           bottom=True)

swagger_init = Resource(library, 'js/swagger-initializer.js',
                        minified='js/swagger-initializer.min.js',
                        depends=(swagger_css, swagger_presets),
                        bottom=True)


@pagelet_config(name='admin', context=Interface, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/admin-index.pt',
                 layer=IAdminLayer)
class AdminPage(AdminView):
    """Main management page"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.environment module

This module is used to display running version of all installed packages.
"""

import os

from pyams_utils.list import unique

try:
    from importlib.metadata import distributions
except ImportError:
    from importlib_metadata import distributions

from pyramid.decorator import reify

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import ViewContentProvider, viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable
from pyams_zmi.table import InnerTableAdminView, MultipleTablesAdminView, NameColumn, Table
from pyams_zmi.zmi.interfaces import IConfigurationMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@viewlet_config(name='environment.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IConfigurationMenu, weight=50,
                permission=MANAGE_SYSTEM_PERMISSION)
class EnvironmentMenu(NavigationMenuItem):
    """Environment menu"""

    label = _("Environment")
    href = '#environment.html'


@pagelet_config(name='environment.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class EnvironmentView(MultipleTablesAdminView):
    """Environment view"""

    table_label = _("Application runtime environment")


#
# Packages versions
#

class PackagesVersionsTable(Table):
    """Packages versions table"""

    def __init__(self, context, request):
        super().__init__(context, request)
        if distributions is not None:
            self.distributions = distributions()
        else:
            environment = self.environment = Environment()
            environment.scan()

    @reify
    def id(self):
        return '{}_packages'.format(super().id)

    @reify
    def values(self):
        if distributions is not None:
            return unique(self.distributions,
                          key=lambda x: x.name)
        return self.environment


@adapter_config(name='package-name',
                required=(ISiteRoot, IAdminLayer, PackagesVersionsTable),
                provides=IColumn)
class PackagesNameColumn(NameColumn):
    """Package name column"""

    i18n_header = _("Package name")
    weight = 10

    def get_value(self, obj):
        """Value getter"""
        if distributions is not None:
            return obj.name
        environment = self.table.values
        return environment[obj][0].project_name


@adapter_config(name='package-version',
                required=(ISiteRoot, IAdminLayer, PackagesVersionsTable),
                provides=IColumn)
class PackageVersionColumn(NameColumn):
    """Package version column"""

    i18n_header = _("Version")
    weight = 20

    def get_value(self, obj):
        """Value getter"""
        return obj.version


@adapter_config(name='package-path',
                required=(ISiteRoot, IAdminLayer, PackagesVersionsTable),
                provides=IColumn)
class PackagePathColumn(NameColumn):
    """Package path column"""

    i18n_header = _("Path")
    weight = 30

    def get_value(self, obj):
        """Value getter"""
        if distributions is not None:
            return str(obj.locate_file(''))
        environment = self.table.values
        return environment[obj][0].location


@adapter_config(name='packages-versions',
                required=(ISiteRoot, IAdminLayer, EnvironmentView),
                provides=IInnerTable)
class PackagesVersionsView(InnerTableAdminView, ViewContentProvider):
    """Packages versions view"""

    table_class = PackagesVersionsTable
    table_label = _("Packages versions")

    hide_section = True

    weight = 10


#
# Environment variables
#

class EnvironmentTable(Table):
    """Environment table"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.environ = os.environ

    @reify
    def id(self):
        return '{}_environ'.format(super().id)

    @reify
    def values(self):
        return self.environ


@adapter_config(name='setting-name',
                required=(ISiteRoot, IAdminLayer, EnvironmentTable),
                provides=IColumn)
class EnvironmentNameColumn(NameColumn):
    """Environment name column"""

    i18n_header = _("Variable name")
    weight = 10

    def get_value(self, obj):
        """Value getter"""
        return obj


@adapter_config(name='environ-value',
                required=(ISiteRoot, IAdminLayer, EnvironmentTable),
                provides=IColumn)
class EnvironmentValueColumn(NameColumn):
    """Environment value column"""

    i18n_header = _("Value")
    weight = 20

    def get_value(self, obj):
        """Value getter"""
        environ = self.table.environ
        return environ.get(obj)


@adapter_config(name='environment-variables',
                required=(ISiteRoot, IAdminLayer, EnvironmentView),
                provides=IInnerTable)
class EnvironmentVariablesView(InnerTableAdminView, ViewContentProvider):
    """Environment view"""

    table_class = EnvironmentTable
    table_label = _("Environment variables")

    hide_section = True

    weight = 20


#
# Configuration settings
#

class ConfigurationTable(Table):
    """Configuration settings table"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.settings = request.registry.settings

    @reify
    def id(self):
        return '{}_config'.format(super().id)

    @reify
    def values(self):
        return self.settings


@adapter_config(name='setting-name',
                required=(ISiteRoot, IAdminLayer, ConfigurationTable),
                provides=IColumn)
class ConfigurationNameColumn(NameColumn):
    """Setting name column"""

    i18n_header = _("Setting name")
    weight = 10

    def get_value(self, obj):
        """Value getter"""
        return obj


@adapter_config(name='setting-value',
                required=(ISiteRoot, IAdminLayer, ConfigurationTable),
                provides=IColumn)
class ConfigurationValueColumn(NameColumn):
    """Setting value column"""

    i18n_header = _("Value")
    weight = 20

    def get_value(self, obj):
        """Value getter"""
        settings = self.table.values
        return settings.get(obj)


@adapter_config(name='configuration-settings',
                required=(ISiteRoot, IAdminLayer, EnvironmentView),
                provides=IInnerTable)
class ConfigurationView(InnerTableAdminView, ViewContentProvider):
    """Packages versions view"""

    table_class = ConfigurationTable
    table_label = _("Configuration settings")

    hide_section = True

    weight = 30



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.utils module

This module provides a small set of generic components.
"""

from zope.location import ILocation

from pyams_utils.adapter import adapter_config, query_adapter
from pyams_utils.interfaces import MISSING_INFO
from pyams_zmi.interfaces import IObjectHint, IObjectIcon, IObjectLabel, IObjectName

__docformat__ = 'restructuredtext'


@adapter_config(required=ILocation,
                provides=IObjectName)
def location_name(context):
    """Basic location name factory"""
    return context.__name__


def get_object_name(context, request, view=None, name=''):
    """Get object name"""
    adapter = query_adapter(IObjectName, request, context, view, name)
    if name and (adapter is None):
        adapter = query_adapter(IObjectName, request, context, view)
    return adapter


@adapter_config(required=ILocation,
                provides=IObjectLabel)
def location_label(context):
    """Basic location name factory"""
    return MISSING_INFO


def get_object_label(context, request, view=None, name=''):
    """Get object label"""
    adapter = query_adapter(IObjectLabel, request, context, view, name)
    if name and (adapter is None):
        adapter = query_adapter(IObjectLabel, request, context, view)
    return adapter


@adapter_config(required=ILocation,
                provides=IObjectIcon)
def location_icon(context):  # pylint: disable=unused-argument
    """Basic location icon factory"""
    return 'far fa-square'


def get_object_icon(context, request, view=None, name=''):
    """Get object icon"""
    adapter = query_adapter(IObjectIcon, request, context, view, name)
    if name and (adapter is None):
        adapter = query_adapter(IObjectIcon, request, context, view)
    return adapter


def get_object_hint(context, request, view=None, name=''):
    """Get object hint"""
    adapter = query_adapter(IObjectHint, request, context, view, name)
    if name and (adapter is None):
        adapter = query_adapter(IObjectHint, request, context, view)
    return adapter



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.form module

This module provides all base form-related classes to be used into PyAMS management
interface.
"""

import json

from pyramid.decorator import reify
from zope.container.interfaces import IContainer
from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.button import Buttons, handler
from pyams_form.form import AddForm, DisplayForm, EditForm
from pyams_form.group import Group, GroupForm, GroupManager
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IForm
from pyams_form.subform import InnerAddForm, InnerDisplayForm, InnerEditForm
from pyams_i18n.schema import II18nField
from pyams_skin.interfaces.view import IInnerPage, IModalAddForm, IModalDisplayForm, IModalEditForm, IModalPage
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config, query_adapter
from pyams_utils.data import ObjectDataManagerMixin
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_utils.traversing import get_parent
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IAddFormButtons, IDisplayFormButtons, IEditFormButtons, \
    IFormGroupChecker, IFormGroupSwitcher, IFormLegend, IFormTitle, IModalAddFormButtons, \
    IModalDisplayFormButtons, IModalEditFormButtons
from pyams_zmi.utils import get_object_hint, get_object_label
from pyams_zmi.view import AdminView

__docformat__ = 'restructuredtext'


class BaseFormMixin:
    """Base form mixin class"""

    @reify
    def title(self):
        """Title getter"""
        title = query_adapter(IFormTitle, self.request, self.context, self)
        if not title:
            title = super().title or MISSING_INFO
        subtitle = getattr(self, 'subtitle', None)
        if subtitle:
            translate = self.request.localizer.translate
            if '<br />' in title:
                title += f'<br /><div class="small pt-2">{translate(subtitle)}</div>'
            elif title.startswith('<'):
                title += f'<br />{translate(subtitle)}'
            else:
                title = TITLE_SPAN_BREAK.format(title, translate(subtitle))
        return title

    _legend = None

    @property
    def legend(self):
        """Legend getter"""
        if self._legend is not None:
            return self._legend
        legend = query_adapter(IFormLegend, self.request, self.context, self)
        return legend or super().legend

    @legend.setter
    def legend(self, value):
        """Legend setter"""
        self._legend = value


@adapter_config(required=(Interface, IAdminLayer, IForm),
                provides=IFormTitle)
def get_form_title(context, request, form):
    """Default form title getter"""
    hint = get_object_hint(context, request, form)
    label = get_object_label(context, request, form, name='form-title')
    if hint and label:
        return TITLE_SPAN_BREAK.format(hint, label)
    return label or MISSING_INFO


#
# Base add forms
#

# pylint: disable=abstract-method
@implementer(IInnerPage)
class AdminAddForm(ObjectDataManagerMixin, BaseFormMixin,
                   GroupForm, AddForm, AdminView):
    """Management add form"""

    @property
    def buttons(self):
        """Default add form buttons getter"""
        if self.mode == DISPLAY_MODE:
            return Buttons(Interface)
        return Buttons(IAddFormButtons)

    @handler(IAddFormButtons['add'])
    def handle_add(self, action):
        """Default add form button handler"""
        super().handle_add(self, action)  # pylint: disable=too-many-function-args


@implementer(IModalAddForm)
class AdminModalAddForm(AdminAddForm):
    """Modal management add form"""

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IModalAddFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])
    modal_content_class = FieldProperty(IModalPage['modal_content_class'])

    ajax_form_target = None

    @handler(IModalAddFormButtons['add'])
    def handle_add(self, action):
        super().handle_add(self, action)  # pylint: disable=too-many-function-args


@implementer(IInnerPage)
class AdminInnerAddForm(ObjectDataManagerMixin, BaseFormMixin,
                        GroupForm, InnerAddForm, AdminView):
    """Inner management add form"""

    buttons = Buttons(Interface)


#
# Base edit forms
#

@implementer(IInnerPage)
class AdminEditForm(ObjectDataManagerMixin, BaseFormMixin,
                    GroupForm, EditForm, AdminView):
    """Management edit form"""

    @property
    def buttons(self):
        """Default inner page buttons getter"""
        if self.mode == DISPLAY_MODE:
            return Buttons(Interface)
        return Buttons(IEditFormButtons)

    @handler(IEditFormButtons['apply'])
    def handle_apply(self, action):
        super().handle_apply(self, action)  # pylint: disable=too-many-function-args


@implementer(IModalEditForm)
class AdminModalEditForm(AdminEditForm):
    """Modal management edit form"""

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IModalEditFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])
    modal_content_class = FieldProperty(IModalPage['modal_content_class'])

    ajax_form_target = None

    @handler(IModalEditFormButtons['apply'])
    def handle_apply(self, action):
        super().handle_apply(self, action)  # pylint: disable=too-many-function-args


@implementer(IInnerPage)
class AdminInnerEditForm(ObjectDataManagerMixin, BaseFormMixin,
                         GroupForm, InnerEditForm, AdminView):
    """Inner management edit form"""

    buttons = Buttons(Interface)


#
# Base display forms
#

@implementer(IInnerPage)
class AdminDisplayForm(ObjectDataManagerMixin, BaseFormMixin,
                       GroupManager, DisplayForm, AdminView):
    """Management display form"""

    buttons = Buttons(IDisplayFormButtons)


@implementer(IModalDisplayForm)
class AdminModalDisplayForm(AdminDisplayForm):
    """Modal management display form"""

    buttons = Buttons(IModalDisplayFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])
    modal_content_class = FieldProperty(IModalPage['modal_content_class'])


@implementer(IInnerPage)
class AdminInnerDisplayForm(ObjectDataManagerMixin, BaseFormMixin,
                            GroupForm, InnerDisplayForm, AdminView):
    """Inner management display form"""

    buttons = Buttons(Interface)


#
# Switcher group
#

@implementer(IFormGroupSwitcher)
class FormGroupSwitcher(ObjectDataManagerMixin, Group):
    """Form group switcher

    A "group switcher" is based on a "switcher" component provided by MyAMS, which allows to
    switch a whole fieldset.
    """

    minus_class = FieldProperty(IFormGroupSwitcher['minus_class'])
    plus_class = FieldProperty(IFormGroupSwitcher['plus_class'])
    switcher_mode = FieldProperty(IFormGroupSwitcher['switcher_mode'])

    @property
    def state(self):
        """Current state getter"""
        if self.switcher_mode == 'always':
            return 'open'
        if self.switcher_mode == 'never':
            return 'closed'
        # else: automatic mode
        for widget in self.widgets.values():
            if widget.ignore_context:
                continue
            field = widget.field
            if self.ignore_context:
                value = field.default
            else:
                context = widget.context
                name = field.getName()
                value = getattr(field.interface(context), name, None)
            if value != field.default:
                if II18nField.providedBy(field):  # pylint: disable=no-value-for-parameter
                    for i18n_value in value.values():
                        if i18n_value:
                            return 'open'
                return 'open'
        return 'closed'


@adapter_config(name="switch_data",
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class FormGroupSwitcherDataTALESExtension(ContextRequestViewAdapter):
    """Form group switcher data TALES extension

    This helper extension is used in a form group template to check if this group
    is a form group switcher .
    """

    def render(self, view=None):
        """Extension renderer"""
        if view is None:
            view = self.view
        switcher = IFormGroupSwitcher(view, None)
        if switcher is None:
            return None
        return json.dumps({
            'ams-minus-class': switcher.minus_class,
            'ams-plus-class': switcher.plus_class,
            'ams-switcher-state': switcher.state
        })


#
# Checker group
#

@implementer(IFormGroupChecker)
class FormGroupChecker(ObjectDataManagerMixin, Group):
    """Form group checker

    A "group checker" is based on a "checker" component provided by MyAMS, which allows to
    hide or disable a whole fieldset with a checkbox matching a form's field value.
    """

    checker_fieldname = FieldProperty(IFormGroupChecker['checker_fieldname'])
    checker_mode = FieldProperty(IFormGroupChecker['checker_mode'])

    def __init__(self, context, request, parent_form):
        super().__init__(context, request, parent_form)
        name, field = next(iter(self.fields.items()))
        self.checker_fieldname = name
        self.legend = field.field.title
        self.fields[self.checker_fieldname].widget_factory = SingleCheckBoxFieldWidget

    @reify
    def checker_widget(self):
        """Checker widget getter"""
        return self.widgets[self.checker_fieldname]

    @property
    def checker_state(self):
        """Checker state getter"""
        return 'on' if 'selected' in self.checker_widget.value else 'off'


#
# Simple form renderers
#

class SimpleAddFormRenderer(ContextRequestViewAdapter):
    """Simple add form renderer"""

    table_factory = None

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                self.table_factory, changes)
            ]
        }


class SimpleEditFormRenderer(ContextRequestViewAdapter):
    """Simple edit form renderer"""

    parent_interface = IContainer
    table_factory = None

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        target = get_parent(self.context, self.parent_interface)
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_refresh_callback(target, self.request,
                                                    self.table_factory, self.context)
            ]
        }



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.controlpanel module

This module defines views and content providers which are used to get access to the local
site manager contents view.
"""

from pyramid.decorator import reify
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.principalannotation.interfaces import IPrincipalAnnotationUtility

from pyams_file.interfaces import IBlobReferenceManager
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.timezone import IServerTimezone
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.table import ITableWithActions
from pyams_zmi.interfaces.viewlet import IControlPanelMenu, IUtilitiesMenu
from pyams_zmi.table import NameColumn, Table, TableAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _  # pylint: disable=ungrouped-import


#
# Generic utilities labels
#

@adapter_config(required=(IBlobReferenceManager, IAdminLayer, Interface),
                provides=IObjectLabel)
def blob_reference_manager_label(context, request, view):
    """Blobs references manager label"""
    return request.localizer.translate(_("Blobs references manager"))


@adapter_config(required=(IIntIds, IAdminLayer, Interface),
                provides=IObjectLabel)
def intids_label(context, request, view):
    """Internal IDs label"""
    return request.localizer.translate(_("Internal IDs"))


@adapter_config(required=(IServerTimezone, IAdminLayer, Interface),
                provides=IObjectLabel)
def timezone_label(context, request, view):
    """Server timezone label getter"""
    return request.localizer.translate(_("Server timezone"))


@adapter_config(required=(IPrincipalAnnotationUtility, IAdminLayer, Interface),
                provides=IObjectLabel)
def principal_annotation_utility_label(context, request, view):
    """Principal annotation utility label getter"""
    return request.localizer.translate(_("Principals annotations"))


@viewletmanager_config(name='utilities.menu', context=ISiteRoot, layer=IAdminLayer,
                       manager=IControlPanelMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION, provides=IUtilitiesMenu)
class UtilitiesMenuItem(NavigationMenuItem):
    """Utilities menu item"""

    label = _("Utilities")
    icon_class = 'fab fa-codepen'
    href = '#utilities.html'


@implementer(ITableWithActions)
class UtilitiesTable(Table):
    """Utilities table list"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes.setdefault('table', {}).setdefault('data-page-length', 25)
        return attributes


@adapter_config(required=(ISiteRoot, IAdminLayer, UtilitiesTable),
                provides=IValues)
class UtilitiesTableValues(ContextRequestViewAdapter):
    """Utilities tables values adapter"""

    @property
    def values(self):
        """Utilities table values"""
        sm = self.context.getSiteManager()  # pylint: disable=invalid-name
        yield from sm.values()


@adapter_config(name='name',
                required=(ISiteRoot, IAdminLayer, UtilitiesTable),
                provides=IColumn)
class UtilityNameColumn(NameColumn):
    """Utility name column"""


@pagelet_config(name='utilities.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class UtilitiesView(TableAdminView):
    """Utilities view"""

    title = _("Control panel")
    table_class = UtilitiesTable
    table_label = _("Site utilities")



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.profile module

This module provides components used for user profile management.
"""

from zope.interface import Interface, implementer

from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_table.interfaces import ITable
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_zmi.form import AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.profile import IUserProfile
from pyams_zmi.interfaces.table import ITableAttributes
from pyams_zmi.zmi.interfaces import IUserProfileEditForm

__docformat__ = 'restructuredtext'

from pyams_zmi import _


@ajax_form_config(name='user-profile.html',
                  layer=IPyAMSLayer)
@implementer(IUserProfileEditForm)
class UserProfileEditForm(AdminModalEditForm):
    """User profile edit form"""

    subtitle = _("User profile")
    legend = _("My user profile")
    modal_class = 'modal-xl'

    @property
    def fields(self):
        fields = Fields(IUserProfile)
        if not IZMIConfiguration(self.request.root).user_bundle_selection:
            fields = fields.omit('zmi_bundle')
        return fields

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        bundle = self.widgets.get('zmi_bundle')
        if bundle is not None:
            bundle.no_value_message = _("Use default theme")


@adapter_config(required=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IFormContent)
def user_profile_edit_form_content(context, request, form):
    """User profile edit form content getter"""
    return IUserProfile(request.principal)


@adapter_config(required=(Interface, IAdminLayer, UserProfileEditForm),
                provides=IFormTitle)
def user_profile_edit_form_title(context, request, form):
    """User profile edit form title"""
    return request.principal.title


@adapter_config(required=(Interface, IAdminLayer, IUserProfileEditForm),
                provides=IAJAXFormRenderer)
class UserProfileEditFormRenderer(AJAXFormRenderer):
    """User profile edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes:
            return {
                'status': 'redirect'
            }
        return super().render(changes)


@adapter_config(name='profile',
                required=(Interface, IAdminLayer, ITable),
                provides=ITableAttributes)
class UserProfileTableAttributesAdapter(ContextRequestViewAdapter):
    """User profile table attributes adapter"""

    def __new__(cls, context, request, view):
        if getattr(request, 'principal', None) is None:
            return None
        return ContextRequestViewAdapter.__new__(cls)

    def update_attributes(self, source: dict):
        """Update source attributes with selected profile length"""
        profile = IUserProfile(self.request, None)
        if profile is not None:
            source.setdefault('table', {}).update({
                'data-page-length': profile.tables_length
            })



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.configuration module

This module provides views and content providers used to handle ZMI configuration.
"""

from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.zmi.interfaces import IConfigurationMenu
from pyams_zmi.interfaces.viewlet import IControlPanelMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@adapter_config(required=(ISiteRoot, IAdminLayer, Interface),
                provides=IObjectLabel)
def site_root_label(context, request, view):
    """Site root label getter"""
    configuration = IZMIConfiguration(context)
    i18n = II18n(configuration)
    return i18n.query_attribute('home_name', request=request) or \
        i18n.query_attribute('site_name', request=request) or \
        request.localizer.translate(_("Home"))


@viewletmanager_config(name='configuration.menu',
                       context=ISiteRoot, layer=IAdminLayer,
                       manager=IControlPanelMenu, weight=10,
                       permission=MANAGE_SYSTEM_PERMISSION,
                       provides=IConfigurationMenu)
class ConfigurationMenu(NavigationMenuItem):
    """Configuration menu"""

    label = _("Configuration")
    icon_class = 'fas fa-sliders-h'


@viewletmanager_config(name='zmi-configuration.menu',
                       context=ISiteRoot, layer=IAdminLayer,
                       manager=IConfigurationMenu, weight=10,
                       permission=MANAGE_SYSTEM_PERMISSION)
class ZMIConfigurationMenu(NavigationMenuItem):
    """ZMI configuration menu"""

    label = _("ZMI configuration")

    href = '#zmi-configuration.html'


@ajax_form_config(name='zmi-configuration.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SYSTEM_PERMISSION)
class ZMIConfigurationForm(AdminEditForm):
    """ZMI configuration form"""

    title = _("ZMI configuration")
    legend = _("Interface configuration")

    fields = Fields(IZMIConfiguration).select('site_name', 'home_name',
                                              'application_name', 'application_package',
                                              'inner_package_name', 'inner_package',
                                              'environment', 'favicon', 'myams_bundle')

    def get_content(self):
        return IZMIConfiguration(self.context)


@adapter_config(required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IAJAXFormRenderer)
class ZMIConfigurationFormRenderer(ContextRequestViewAdapter):
    """ZMI configuration AJAX form renderer"""

    def render(self, changes):
        """ZMI configuration form renderer"""
        if not changes:
            return {
                'status': 'info',
                'message': self.request.localizer.translate(self.view.no_changes_message)
            }
        return {
            'status': 'redirect'
        }


@adapter_config(name='zmi-profile',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationProfileGroup(FormGroupChecker):
    """ZMI configuration profile fields"""

    fields = Fields(IZMIConfiguration).select('user_bundle_selection', 'user_bundles')

    weight = 5


@adapter_config(name='zmi-header',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationHeaderGroup(FormGroupChecker):
    """ZMI configuration header fields"""

    fields = Fields(IZMIConfiguration).select('include_header', 'fixed_header', 'logo')
    # fields['fixed_header'].widget_factory = SingleCheckBoxFieldWidget

    weight = 10


@adapter_config(name='zmi-search',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationSearchGroup(FormGroupChecker):
    """ZMI configuration search fields"""

    fields = Fields(IZMIConfiguration).select('include_site_search', 'site_search_placeholder',
                                              'site_search_handler')

    weight = 20


@adapter_config(name='zmi-navigation',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationNavigationGroup(FormGroupChecker):
    """ZMI configuration navigation fields"""

    fields = Fields(IZMIConfiguration).select('include_menus', 'include_minify_button',
                                              'fixed_navigation', 'accordion_menus')
    fields['include_minify_button'].widget_factory = SingleCheckBoxFieldWidget
    fields['fixed_navigation'].widget_factory = SingleCheckBoxFieldWidget
    fields['accordion_menus'].widget_factory = SingleCheckBoxFieldWidget

    weight = 30


@viewlet_config(name='zmi-navigation-warning', context=ISiteRoot,
                layer=IAdminLayer, view=ZMIConfigurationNavigationGroup,
                manager=IHelpViewletManager)
class NavigationAlertMessage(AlertMessage):
    """Navigation alert message"""

    status = 'danger'
    icon_class = 'fas fa-exclamation-triangle'
    header = _("Warning!")
    _message = _("""Removing navigation menus will also remove access to this form...""")


@adapter_config(name='zmi-ribbon',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationRibbonGroup(FormGroupChecker):
    """ZMI configuration ribbon fields"""

    fields = Fields(IZMIConfiguration).select('include_ribbon', 'fixed_ribbon')
    fields['fixed_ribbon'].widget_factory = SingleCheckBoxFieldWidget

    weight = 40


@adapter_config(name='zmi-misc',
                required=(ISiteRoot, IAdminLayer, ZMIConfigurationForm),
                provides=IGroup)
class ZMIConfigurationMiscGroup(Group):
    """ZMI configuration misc fields"""

    legend = _("Custom extensions")

    fields = Fields(IZMIConfiguration).select('custom_stylesheet', 'base_body_css_class',
                                              'custom_script')

    weight = 50



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.profile module

This module provides custom ZMI-related user settings.
"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from pyramid.authorization import ALL_PERMISSIONS, Allow, Everyone
from pyramid.threadlocal import get_current_request
from zope.container.contained import Contained
from zope.interface import Interface
from zope.intid import IIntIds
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_file.image import get_image_selection
from pyams_file.property import FileProperty
from pyams_security.interfaces.base import IPrincipalInfo, PUBLIC_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_ID
from pyams_utils.adapter import ContextRequestAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_zmi.interfaces.profile import IUserProfile, USER_PROFILE_KEY


@factory_config(IUserProfile)
class UserProfile(Persistent, Contained):
    """User profile persistent class"""

    principal_id = None

    avatar = FileProperty(IUserProfile['avatar'])
    zmi_bundle = FieldProperty(IUserProfile['zmi_bundle'])
    tables_length = FieldProperty(IUserProfile['tables_length'])

    def __acl__(self):
        return [
            (Allow, ADMIN_USER_ID, ALL_PERMISSIONS),
            (Allow, self.principal_id, ALL_PERMISSIONS),
            (Allow, Everyone, PUBLIC_PERMISSION)
        ]

    def get_avatar(self, selection='square', size='32x32', request=None):
        """Avatar URL getter"""
        if self.avatar:
            return get_image_selection(self.avatar, selection, size, request)
        return None


@adapter_config(required=IPrincipalInfo,
                provides=IUserProfile)
def principal_user_profile_factory(principal):
    """Principal user profile factory adapter"""

    def user_profile_callback(profile):
        profile.principal_id = principal.id
        request = get_current_request()
        if request is not None:
            root = request.root
            intids = get_utility(IIntIds)
            locate(profile, root)  # avoid NotYet exception
            locate(profile, root, '++profile++{0}'.format(intids.register(profile)))

    return get_annotation_adapter(principal, USER_PROFILE_KEY, IUserProfile,
                                  locate=False, callback=user_profile_callback)


@adapter_config(required=Interface,
                provides=IUserProfile)
def user_profile_factory(context):
    request = check_request()
    return IUserProfile(request.principal)


@adapter_config(name='profile',
                required=(Interface, Interface),
                provides=ITraversable)
class UserProfileTraverser(ContextRequestAdapter):
    """++profile++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        if not name:
            return IUserProfile(self.request.principal)
        intids = get_utility(IIntIds)
        profile = intids.queryObject(int(name))
        return IUserProfile(profile, None)


@adapter_config(name='zmi_profile',
                required=(Interface, Interface),
                provides=ITALESExtension)
class UserProfileExtension(ContextRequestAdapter):
    """zmi_profile TALES extension"""

    def render(self, request=None):
        if request is None:
            request = self.request
        return IUserProfile(request)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.userlinks module

This module provides "pyams.userlinks" viewlet manager as well as other components which are
used to add links on top of PyAMS management interface.
"""

from pyams_security.interfaces.names import UNKNOWN_PRINCIPAL_ID
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_template.template import template_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.profile import IUserProfile
from pyams_zmi.interfaces.viewlet import IPageHeaderViewletManager, IUserLinksViewletManager, \
    IUserMenuViewletManager


__docformat__ = 'restructuredtext'

from pyams_zmi import _  # pylint: disable=ungrouped-imports


@viewletmanager_config(name='pyams.userlinks', layer=IAdminLayer,
                       manager=IPageHeaderViewletManager, weight=900,
                       provides=IUserLinksViewletManager)
@template_config(template='templates/user-links.pt')
class UserLinksViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """User links viewlet manager"""


@viewletmanager_config(name='pyams.usermenu', layer=IAdminLayer,
                       manager=IUserLinksViewletManager, weight=900,
                       provides=IUserMenuViewletManager)
@template_config(template='templates/user-menu.pt')
class UserMenuViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """User menu viewlet manager"""

    def __new__(cls, context, request, view, manager=None):  # pylint: disable=unused-argument
        principal = request.principal
        if principal.id == UNKNOWN_PRINCIPAL_ID:
            return None
        return WeightOrderedViewletManager.__new__(cls)

    @property
    def profile_icon(self):
        """Profile icon getter"""
        result = None
        profile = IUserProfile(self.request, None)
        if profile is not None:
            result = profile.get_avatar()
        return result or '/--static--/myams/img/profile.png'


@viewlet_config(name='username.menu', layer=IAdminLayer,
                manager=IUserMenuViewletManager, weight=1)
class UserNameMenuItem(MenuItem):
    """Logged in user name menu"""

    href = 'user-profile.html'
    modal_target = True

    css_class = 'bg-primary text-white'
    icon_class = 'fa fa-user-circle'

    @property
    def label(self):
        """Label getter"""
        return '<strong>{}</strong>'.format(self.request.principal.title)


@viewlet_config(name='logout.divider', layer=IAdminLayer,
                manager=IUserMenuViewletManager, weight=999)
class LogoutMenuDivider(MenuDivider):
    """Logout menu divider"""


@viewlet_config(name='logout.menu', layer=IAdminLayer,
                manager=IUserMenuViewletManager, weight=1000)
class LogoutMenuItem(MenuItem):
    """Logout menu item"""

    icon_class = 'fa fa-sign-out-alt'
    label = _("Logout")
    href = 'logout'



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.viewlet.menu module

This module provides components and content providers which are used to build PyAMS
navigation menu.
"""

from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_template.template import template_config
from pyams_utils.data import ObjectDataManagerMixin
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import Viewlet
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IControlPanelMenu, IMenuHeader, \
    INavigationMenu, INavigationMenuItem, INavigationViewletManager, ISiteManagementMenu


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@template_config(template='templates/nav-menu.pt')
@implementer(INavigationMenu)
class NavigationMenu(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Navigation menu"""

    _header = FieldProperty(INavigationMenu['header'])

    @property
    def header(self):
        """Menu header getter"""
        header = queryMultiAdapter((self.context, self.request, self.view, self), IMenuHeader)
        return header or self._header


@template_config(template='templates/nav-item.pt')
@implementer(INavigationMenuItem)
class NavigationMenuItem(ObjectDataManagerMixin, TemplateBasedViewletManager,
                         WeightOrderedViewletManager):
    """Navigation menu item"""

    @property
    def render_empty(self):
        """Getter to check if empty menu should be rendered"""
        return bool(self.href or self.click_handler)

    css_class = FieldProperty(INavigationMenuItem['css_class'])
    icon_class = FieldProperty(INavigationMenuItem['icon_class'])
    label = FieldProperty(INavigationMenuItem['label'])
    badge = FieldProperty(INavigationMenuItem['badge'])
    badge_status = FieldProperty(INavigationMenuItem['badge_status'])
    href = FieldProperty(INavigationMenuItem['href'])
    click_handler = FieldProperty(INavigationMenuItem['click_handler'])
    target = FieldProperty(INavigationMenuItem['target'])
    modal_target = FieldProperty(INavigationMenuItem['modal_target'])

    def get_href(self):
        """Menu item URL getter"""
        return self.href


@template_config(template='templates/nav-header.pt')
class NavigationMenuHeaderDivider(Viewlet):
    """Navigation menu internal header"""


@template_config(template='templates/nav-divider.pt')
class NavigationMenuDivider(Viewlet):
    """Navigation menu divider"""


@viewletmanager_config(name='content-manager.menu', layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=100,
                       provides=IContentManagementMenu)
class ContentManagementMenu(NavigationMenu):
    """Content management menu"""

    _header = _("Content management")


@viewletmanager_config(name='site-manager.menu', layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=200,
                       provides=ISiteManagementMenu)
class SiteManagementMenu(NavigationMenu):
    """Site management menu"""

    _header = _("Site management")


@viewletmanager_config(name='control-panel.menu', layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=300,
                       provides=IControlPanelMenu)
class ControlPanelMenu(NavigationMenu):
    """Control panel menu"""

    _header = _("Control panel")



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.header module

This module provides the "pyams.page_header" viewlet manager, which is used to build the
top panel of PyAMS management interface.
"""

from zope.component import queryMultiAdapter
from zope.interface import Interface

from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_template.template import get_view_template, template_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import EmptyContentProvider, Viewlet, viewlet_config
from pyams_zmi.interfaces import IAdminLayer, IPageTitle
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.interfaces.viewlet import IPageHeaderViewletManager
from pyams_zmi.utils import get_object_label


__docformat__ = 'restructuredtext'

from pyams_zmi import _  # pylint: disable=ungrouped-imports


@viewletmanager_config(name='pyams.page_header', layer=IAdminLayer,
                       provides=IPageHeaderViewletManager)
@template_config(template='templates/page-header.pt')
class PageHeaderViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Page header viewlet manager"""

    def __new__(cls, context, request, view):  # pylint: disable=unused-argument
        configuration = IZMIConfiguration(request.root, None)
        if (configuration is None) or not configuration.include_header:
            return EmptyContentProvider.__new__(EmptyContentProvider)
        return WeightOrderedViewletManager.__new__(cls)


@viewlet_config(name='pyams.content_header', layer=IAdminLayer,
                manager=IHeaderViewletManager, weight=10)
@template_config(template='templates/header.pt')
@template_config(template='templates/modal-header.pt', name='modal')
class ContentHeaderViewlet(Viewlet):
    """Content header viewlet"""

    _title = _("PyAMS admin page")

    modal_template = get_view_template(name='modal')

    @property
    def title(self):
        """Title getter"""
        title = queryMultiAdapter((self.context, self.request, self.view), IPageTitle)
        if title is None:
            title = IPageTitle(self.context, None)
        return title or self._title

    def render(self):
        if IModalPage.providedBy(self.view):
            return self.modal_template()
        return super().render()


def get_admin_view_title(context, request, view, label=None):
    """Shared tool view title getter"""
    translate = request.localizer.translate
    object_label = get_object_label(context, request, view)
    if label is None:
        label = getattr(view, 'header_label', None)
    if not label:
        return object_label
    return f'{object_label} <small><small>' \
           f' <i class="px-2 fas fa-chevron-right"></i> ' \
           f'{translate(label)}</small></small>'


@adapter_config(required=(Interface, IAdminLayer, Interface),
                provides=IPageTitle)
def admin_view_title(context, request, view):
    """Base admin view title adapter"""
    return get_admin_view_title(context, request, view)


@adapter_config(required=(Interface, IAdminLayer, IModalPage),
                provides=IPageTitle)
def get_modal_admin_view_title(context, request, view):
    """Modal admin view title adapter"""
    return get_object_label(request.root, request, view)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.swagger module

This module defines components which can be used to access Swagger-UI interface.
"""

from zope.interface import implementer

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.view import IInnerPage
from pyams_template.template import template_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.interfaces import IConfigurationMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@viewlet_config(name='swagger-ui.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IConfigurationMenu, weight=100,
                permission=VIEW_SYSTEM_PERMISSION)
class SwaggerMenu(NavigationMenuItem):
    """Swagger-UI menu"""

    label = _("Swagger API")
    href = '#swagger-ui.html'


@pagelet_config(name='swagger-ui.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/swagger.pt', layer=IPyAMSLayer)
@implementer(IInnerPage)
class SwaggerView:
    """Swagger view"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.rest module

This module defines PyAMS API documentation service.
"""

from cornice import Service
from cornice.service import get_services
from cornice_swagger import CorniceSwagger

from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.rest import handle_cors_headers


__docformat__ = 'restructuredtext'


#
# Swagger API documentation access
#

swagger = Service(name='OpenAPI',
                  path='/__api__',
                  description="OpenAPI documentation")


@swagger.options(permission=VIEW_SYSTEM_PERMISSION)
def openapi_options(request):
    """OpenAPI OPTIONS verb handler"""
    return handle_cors_headers(request)


@swagger.get(permission=VIEW_SYSTEM_PERMISSION)
def openapi_specification(request):  # pylint: disable=unused-argument
    """OpenAPI specification"""
    doc = CorniceSwagger(get_services())
    doc.summary_docstrings = True
    return doc.generate('PyAMS', '1.0')



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces module

Public interfaces module.
"""

from zope.interface import Interface
from zope.schema import TextLine

from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.view import IFullPage, IInnerPage
from pyams_template.template import layout_config
from pyams_viewlet.interfaces import IViewletManager


class IAdminLayer(IPyAMSLayer):
    """PyAMS management layer"""


PYAMS_ADMIN_SKIN_NAME = 'PyAMS admin skin'

TITLE_SPAN = '<span class="tiny">{}</span>'
TITLE_SPAN_BREAK = '<span class="tiny">{}</span><br />{}'

SMALL_TITLE_SPAN = '<span class="small">{}</span>'
SMALL_TITLE_SPAN_BREAK = '<span class="small">{}</span><br />{}'


@layout_config(template='templates/admin-layout.pt')
class IAdminView(IFullPage):
    """Management view marker interface"""

    header_label = TextLine(title="Header label")


class IInnerAdminView(IInnerPage, IAdminView):
    """Inner management view marker interface"""

    title = TextLine(title="View title")


class ICompositeView(IViewletManager):
    """Composite view marker interface"""


class IPageTitle(Interface):
    """Simple page title interface"""


class IObjectName(Interface):
    """Object name getter interface"""


class IObjectLabel(Interface):
    """Object label getter interface"""


class IObjectIcon(Interface):
    """Object icon getter interface"""


class IObjectHint(Interface):
    """Object hint getter interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.profile module

This module defines the interfaces which are required to handle ZMI-related settings
in user profile.
"""

from zope.annotation import IAttributeAnnotatable
from zope.schema import Choice

from pyams_file.schema import ThumbnailImageField
from pyams_zmi.interfaces.configuration import USER_BUNDLES_VOCABULARY


__docformat__ = 'restructuredtext'

from pyams_zmi import _


USER_PROFILE_KEY = 'pyams_zmi.profile'


class IUserProfile(IAttributeAnnotatable):
    """User public profile preferences"""

    avatar = ThumbnailImageField(title=_("Profile's avatar"),
                                 description=_("This picture will be associated to your user "
                                               "profile"),
                                 required=False)

    zmi_bundle = Choice(title=_("Graphical theme"),
                        description=_("You can choose a custom theme between those "
                                      "provided in this selection list"),
                        vocabulary=USER_BUNDLES_VOCABULARY,
                        required=False)

    tables_length = Choice(title=_("Default tables length"),
                           description=_("Default length of tables displayed by management "
                                         "interface"),
                           values=(10, 25, 50, 100),
                           default=10)

    def get_avatar(self, selection='square', size='48x48'):
        """Get URL of avatar thumbnail for given selection and size"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.toolbar module

This module provides several viewlet managers, which are used to display a toolbar and
contextual actions on content views.
"""

from zope.interface import Interface

from pyams_skin.interfaces.view import IModalPage
from pyams_skin.viewlet.actions import ContextActionsViewletManager
from pyams_skin.viewlet.menu import DropdownMenu
from pyams_template.template import template_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import EmptyContentProvider
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextActionsDropdownMenu, IContextAddingsViewletManager, \
    IViewWithoutToolbar, ISecondaryActionsViewletManager, IToolbarViewletManager


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@viewletmanager_config(name='pyams.toolbar',
                       layer=IAdminLayer, view=Interface,
                       provides=IToolbarViewletManager)
@template_config(template='templates/toolbar.pt')
class ToolbarViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Actions viewlet manager"""


@viewletmanager_config(name='pyams.toolbar',
                       layer=IAdminLayer, view=IModalPage,
                       provides=IToolbarViewletManager)
class ModalToolbarViewletManager(ToolbarViewletManager):
    """Modal page actions viewlet manager"""

    def _get_viewlets(self):
        return ()
    
    
@viewletmanager_config(name='pyams.toolbar',
                       layer=IAdminLayer, view=IViewWithoutToolbar,
                       provides=IToolbarViewletManager)
class MissingToolbarViewletManager(EmptyContentProvider):
    """Missing toolbar viewlet manager"""
    

@viewletmanager_config(name='pyams.context_addings',
                       layer=IAdminLayer, view=Interface,
                       manager=IToolbarViewletManager, weight=10,
                       provides=IContextAddingsViewletManager)
class AddingsViewletManager(DropdownMenu):
    """Custom addings menu"""

    label = _("Add...")
    status = 'primary'
    css_class = 'btn-sm'
    icon_class = 'fas fa-plus'


@viewletmanager_config(name='pyams.actions',
                       layer=IAdminLayer, view=Interface,
                       manager=IToolbarViewletManager, weight=999,
                       provides=IContextActionsDropdownMenu)
class ContextActionsDropdownMenu(DropdownMenu):
    """Custom context actions menu"""

    label = _("Actions")
    css_class = 'btn-sm'


@viewletmanager_config(name='pyams.secondary-actions',
                       layer=IAdminLayer, view=Interface,
                       manager=IToolbarViewletManager, weight=980,
                       provides=ISecondaryActionsViewletManager)
class SecondaryActionsViewletManager(ContextActionsViewletManager):
    """Secondary actions viewlet manager"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.helper.event module

This module provides a few set of helpers which can be used to render JSON results which
can be used by MyAMS to refresh elements.
"""

from datetime import datetime, timezone

from zope.component import getMultiAdapter
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import alsoProvides

from pyams_form.interfaces.form import IContextAware, IFormAware
from pyams_form.interfaces.widget import IFieldWidget
from pyams_form.util import expand_prefix
from pyams_utils.factory import get_object_factory
from pyams_utils.url import absolute_url
from pyams_zmi.table import get_table_id

__docformat__ = 'restructuredtext'


def get_json_image_refresh_callback(image, image_id, request):
    """Get image refresh callback settings"""
    dc = IZopeDublinCore(image, None)  # pylint: disable=invalid-name
    if dc is None:
        timestamp = datetime.now(timezone.utc).timestamp()
    else:
        timestamp = dc.modified.timestamp()  # pylint: disable=no-member
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.refreshImage',
        'options': {
            'image_id': image_id,
            'src': '{}?_={}'.format(absolute_url(image, request), timestamp)
        }
    }


def get_json_widget_refresh_callback(form, field_name, request=None):
    """Get widget refresh callback settings"""
    field = form.fields[field_name]
    factory = field.widget_factory.get(form.mode)
    if request is None:
        request = form.request
    if factory is not None:
        widget = factory(field.field, request)
    else:
        widget = getMultiAdapter((field.field, request), IFieldWidget)
    widget.name = expand_prefix(form.prefix) + expand_prefix(form.widgets.prefix) + field.__name__
    widget.id = widget.name.replace('.', '-')
    widget.form = form
    widget.context = form.get_content()
    widget.mode = form.mode
    widget.ignore_request = True
    widget.ignore_context = False
    alsoProvides(widget, IFormAware)
    alsoProvides(widget, IContextAware)
    widget.update()
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.refreshWidget',
        'options': {
            'widget_id': widget.id,
            'content': widget()
        }
    }


def get_json_table_refresh_callback(context, request, table_factory):
    """Get table refresh callback settings"""
    factory = get_object_factory(table_factory)
    table = factory(context, request)
    table.update()
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.refreshTable',
        'options': {
            'table_id': get_table_id(table),
            'content': table.render()
        }
    }


def get_json_table_row_add_callback(context, request, table_factory, item):
    """Get table row add callback settings"""
    factory = get_object_factory(table_factory)
    table = factory(context, request)
    table.update()
    row = table.setup_row(item)
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.addTableRow',
        'options': {
            'table_id': get_table_id(table),
            'row_id': table.get_row_id(item),
            'content': table.render_row(row)
        }
    }


def get_json_table_row_refresh_callback(context, request, table_factory, item):
    """Get table row refresh callback settings"""
    factory = get_object_factory(table_factory)
    table = factory(context, request)
    table.update()
    row = table.setup_row(item)
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.refreshTableRow',
        'options': {
            'row_id': table.get_row_id(item),
            'data': table.render_row(row)
        }
    }


def get_json_table_row_delete_callback(context, request, table_factory, item):
    """Get table row delete callback settings"""
    factory = get_object_factory(table_factory)
    table = factory(context, request)
    table.update()
    return {
        'module': 'helpers',
        'callback': 'MyAMS.helpers.deleteTableRow',
        'options': {
            'row_id': table.get_row_id(item)
        }
    }



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.viewlet module

This module defines public interfaces of several content providers which are used
in PyAMS management interface.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface
from zope.schema import Bool, Choice, TextLine

from pyams_skin.interfaces import BOOTSTRAP_STATUS
from pyams_skin.interfaces.viewlet import IDropdownMenu
from pyams_viewlet.interfaces import IViewletManager


class IPageHeaderViewletManager(IViewletManager):
    """Header viewlet marker interface"""


class ITopLinksViewletManager(IViewletManager):
    """Top links viewlet marker interface"""


class IUserLinksViewletManager(IViewletManager):
    """User links viewlet marker interface"""


class IUserMenuViewletManager(IViewletManager):
    """User menu viewlet marker interface"""


class ILeftAsideViewletManager(IViewletManager):
    """Left aside viewlet marker interface"""


class INavigationViewletManager(IViewletManager):
    """Navigation viewlet marker interface"""


class IMenuHeader(Interface):
    """Menu header"""


class INavigationMenu(IViewletManager):
    """Navigation menu viewlet interface"""

    header = TextLine(title="Menu header label",
                      description="Can be customized using an IMenuHeader multi-adapter")


class IContentManagementMenu(INavigationMenu):
    """Content management menu marker interface"""


class IPropertiesMenu(INavigationMenu):
    """Content properties menu"""


class ISiteManagementMenu(INavigationMenu):
    """Site management menu marker interface"""


class IControlPanelMenu(INavigationMenu):
    """Control panel menu marker interface"""


class IUtilitiesMenu(INavigationMenu):
    """Local utilities menu"""


class INavigationMenuItem(IViewletManager):
    """Navigation menu item viewlet interface"""

    label = TextLine(title="Main menu item label")

    css_class = TextLine(title="Main CSS class")

    icon_class = TextLine(title="FontAwesome class of menu item icon, including prefix "
                                "('fa', 'fab'...)")

    badge = TextLine(title="Optional badge text")

    badge_status = Choice(title="Badge status",
                          values=BOOTSTRAP_STATUS,
                          default='info')

    href = TextLine(title="Link target URL",
                    default='#')

    def get_href(self):
        """Complete HREF attribute getter"""

    click_handler = TextLine(title="Optional Javascript click handler name")

    target = TextLine(title="Menu link target name")

    modal_target = Bool(title="Link to modal dialog?",
                        default=False)


class IViewWithoutToolbar(Interface):
    """View without toolbar marker interface"""
    
    
class IToolbarViewletManager(IViewletManager):
    """Toolbar viewlet manager"""


class IContextAddingsViewletManager(IDropdownMenu):
    """Context addings viewlet manager"""


class IContextActionsDropdownMenu(IViewletManager):
    """Custom actions menu viewlet manager"""


class ISecondaryActionsViewletManager(IViewletManager):
    """Custom secondary actions viewlet manager"""
